@@ -66,24 +66,24 @@ EEL_CLASS_BOILERPLATE (NautilusDesktopLinkMonitor,
 		       nautilus_desktop_link_monitor,
 		       G_TYPE_OBJECT)
 
-static NautilusDesktopLinkMonitor *link_monitor = NULL;
+static NautilusDesktopLinkMonitor *the_link_monitor = NULL;
 
 static void
 destroy_desktop_link_monitor (void)
 {
-	if (link_monitor != NULL) {
-		g_object_unref (link_monitor);
+	if (the_link_monitor != NULL) {
+		g_object_unref (the_link_monitor);
 	}
 }
 
 NautilusDesktopLinkMonitor *
 nautilus_desktop_link_monitor_get (void)
 {
-	if (link_monitor == NULL) {
-		link_monitor = NAUTILUS_DESKTOP_LINK_MONITOR (g_object_new (NAUTILUS_TYPE_DESKTOP_LINK_MONITOR, NULL));
+	if (the_link_monitor == NULL) {
+		g_object_new (NAUTILUS_TYPE_DESKTOP_LINK_MONITOR, NULL);
 		eel_debug_call_at_shutdown (destroy_desktop_link_monitor);
 	}
-	return link_monitor;
+	return the_link_monitor;
 }
 
 void
@@ -111,18 +111,59 @@ nautilus_desktop_link_monitor_delete_link (NautilusDesktopLinkMonitor *monitor,
 	}
 }
 
+static gboolean
+volume_file_name_used (NautilusDesktopLinkMonitor *monitor,
+		       const char *name)
+{
+	GList *l;
+	char *other_name;
+	gboolean same;
+
+	for (l = monitor->details->volume_links; l != NULL; l = l->next) {
+		other_name = nautilus_desktop_link_get_file_name (l->data);
+		same = strcmp (name, other_name) == 0;
+		g_free (other_name);
+
+		if (same) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+char *
+nautilus_desktop_link_monitor_make_filename_unique (NautilusDesktopLinkMonitor *monitor,
+						    const char *filename)
+{
+	char *unique_name;
+	int i;
+	
+	i = 2;
+	unique_name = g_strdup (filename);
+	while (volume_file_name_used (monitor, unique_name)) {
+		g_free (unique_name);
+		unique_name = g_strdup_printf ("%s.%d", filename, i++);
+	}
+	return unique_name;
+}
+
 static void
 create_volume_link (NautilusDesktopLinkMonitor *monitor,
 		    GnomeVFSVolume *volume)
 {
 	NautilusDesktopLink *link;
-	
+
+	link = NULL;
+
 	if (!gnome_vfs_volume_is_user_visible (volume)) {
 		return;
 	}
 
-	link = nautilus_desktop_link_new_from_volume (volume);
-	monitor->details->volume_links = g_list_prepend (monitor->details->volume_links, link);
+	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_DESKTOP_VOLUMES_VISIBLE)) {
+		link = nautilus_desktop_link_new_from_volume (volume);
+		monitor->details->volume_links = g_list_prepend (monitor->details->volume_links, link);
+	}
 }
 
 
@@ -220,6 +261,30 @@ desktop_trash_visible_changed (gpointer callback_data)
 	}
 }
 
+static void
+desktop_volumes_visible_changed (gpointer callback_data)
+{
+	GnomeVFSVolumeMonitor *volume_monitor;
+	NautilusDesktopLinkMonitor *monitor;
+	GList *l, *volumes;
+	
+	volume_monitor = gnome_vfs_get_volume_monitor ();
+	monitor = NAUTILUS_DESKTOP_LINK_MONITOR (callback_data);
+
+	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_DESKTOP_VOLUMES_VISIBLE) && monitor->details->volume_links == NULL) {
+		volumes = gnome_vfs_volume_monitor_get_mounted_volumes (volume_monitor);
+		for (l = volumes; l != NULL; l = l->next) {
+			create_volume_link (monitor, l->data);
+			gnome_vfs_volume_unref (l->data);
+		}
+		g_list_free (volumes);
+	} else {
+		g_list_foreach (monitor->details->volume_links, (GFunc)g_object_unref, NULL);
+		g_list_free (monitor->details->volume_links);
+		monitor->details->volume_links = NULL;
+	}
+}
+
 static void
 nautilus_desktop_link_monitor_init (gpointer object, gpointer klass)
 {
@@ -230,6 +295,8 @@ nautilus_desktop_link_monitor_init (gpointer object, gpointer klass)
 
 	monitor = NAUTILUS_DESKTOP_LINK_MONITOR (object);
 
+	the_link_monitor = monitor;
+	
 	monitor->details = g_new0 (NautilusDesktopLinkMonitorDetails, 1);
 
 	/* We keep around a ref to the desktop dir */
@@ -248,6 +315,7 @@ nautilus_desktop_link_monitor_init (gpointer object, gpointer klass)
 	}
 
 	volume_monitor = gnome_vfs_get_volume_monitor ();
+	
 	volumes = gnome_vfs_volume_monitor_get_mounted_volumes (volume_monitor);
 	for (l = volumes; l != NULL; l = l->next) {
 		volume = l->data;
@@ -265,8 +333,10 @@ nautilus_desktop_link_monitor_init (gpointer object, gpointer klass)
 	eel_preferences_add_callback (NAUTILUS_PREFERENCES_DESKTOP_TRASH_VISIBLE,
 				      desktop_trash_visible_changed,
 				      monitor);
+	eel_preferences_add_callback (NAUTILUS_PREFERENCES_DESKTOP_VOLUMES_VISIBLE,
+				      desktop_volumes_visible_changed,
+				      monitor);
 
-	
 	monitor->details->mount_id = g_signal_connect_object (volume_monitor, "volume_mounted",
 							      G_CALLBACK (volume_mounted_callback), monitor, 0);
 	monitor->details->unmount_id = g_signal_connect_object (volume_monitor, "volume_unmounted",
@@ -312,6 +382,9 @@ desktop_link_monitor_finalize (GObject *object)
 	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_DESKTOP_TRASH_VISIBLE,
 					 desktop_trash_visible_changed,
 					 monitor);
+	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_DESKTOP_VOLUMES_VISIBLE,
+					 desktop_volumes_visible_changed,
+					 monitor);
 
 	if (monitor->details->mount_id != 0) {
 		g_source_remove (monitor->details->mount_id);
