@@ -73,12 +73,21 @@ GList *              nautilus_application_get_window_list           (void);
 GList *              nautilus_application_get_spatial_window_list    (void);
 
 NautilusWindow *     nautilus_application_present_spatial_window     (NautilusApplication *application,
+								      NautilusWindow      *requesting_window,
 								      const char          *location,
 								      GdkScreen           *screen);
+NautilusWindow *     nautilus_application_present_spatial_window_with_selection (NautilusApplication *application,
+										 NautilusWindow      *requesting_window,
+										 const char          *location,
+										 GList		     *new_selection,
+										 GdkScreen           *screen);
+
 NautilusWindow *     nautilus_application_create_navigation_window     (NautilusApplication *application,
 									GdkScreen           *screen);
+
 void                 nautilus_application_close_all_navigation_windows (void);
 void                 nautilus_application_close_parent_windows     (NautilusSpatialWindow *window);
+void                 nautilus_application_close_all_spatial_windows  (void);
 void                 nautilus_application_open_desktop      (NautilusApplication *application);
 void                 nautilus_application_close_desktop     (void);
 
